# coding: utf-8
#
# Copyright 2018 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#                   Robert Schenk, <robert.schenk@uni-leipzig.de>#
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>
"""
VDEH, refs #9868.

Note: These tasks are based on an MABxml version of the original binary MAB.

External Java app required (https://sourceforge.net/projects/dnb-conv-tools/):

    $ sha1sum Mab2Mabxml-1.9.9.jar
    b37c4663fe6e4dcc55e71253266516e417ec9c44  Mab2Mabxml-1.9.9.jar

Configuration:

[vdeh]

input = /path/to/mab (ask RS)
Mab2Mabxml.jar = /path/to/local/copy.jar
index-url = http://localhost:8983/solr/biblio/select

"""

import datetime

import luigi

from gluish.intervals import weekly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.task import DefaultTask


class VDEHTask(DefaultTask):
    """
    This task inherits functionality from `siskin.task.DefaultTask`.
    """
    TAG = '130'

    def closest(self):
        return weekly(date=self.date)


class VDEHXML(VDEHTask):
    """
    Convert binary MAB to MABxml.

    Requires Mab2Mabxml, downloadable from
    https://sourceforge.net/projects/dnb-conv-tools/.
    """

    def run(self):
        output = shellout("java -jar {jarfile} -i {input} -o {output}",
                          jarfile=self.config.get("vdeh", "Mab2Mabxml.jar"),
                          input=self.config.get("vdeh", "input"))
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext="xml"))


class VDEHRemoveIllegalChars(VDEHTask):
    """
    Remove Nichtsortierzeichen.

    https://stackoverflow.com/a/7774512, https://stackoverflow.com/a/36371662

    We want to remove unicode codepoint U+00AC (c2ac, o302 o254), aka the
    "NOT SIGN", aka the "Nichtsortierzeichen".

    When removing only o254, the dangling o302 causes problems (invalid
    continuation byte) - but not always.

    tr -d '[:cntrl:]' would not remove it, since c2ac starts with 0302,
    which is used for control chars as well (https://is.gd/GRIjmy).

    s/\u00AC\u00C2//g would not work for some reason.

    Anecdata: The stripped file is 57740 bytes smaller.
    """

    def requires(self):
        return VDEHXML()

    def run(self):
        output = shellout(r" sed -e $(echo -e 's/\o302\o254//g') < {input} > {output} ", input=self.input().path)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='xml'))


class VDEHMARC(VDEHTask):
    """ Convert MABxml to BinaryMarc """

    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return VDEHRemoveIllegalChars()

    def run(self):
        output = shellout("""python {script} {input} {output} {server}""",
                          script=self.assets("130/130_marcbinary.py"),
                          input=self.input().path,
                          server=self.config.get('vdeh', 'index-url'))
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='fincmarc.mrc'))



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,R0904,E1103,C0301
#
# Copyright 2019 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#                   Robert Schenk, <robert.schenk@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>
"""

Source: Gallica (Bibliothèque nationale de France, BnF)
SID: 20
Ticket: #14793
Origin: OAI
Updates: monthly

"""

import datetime

import luigi

from gluish.intervals import monthly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.task import DefaultTask


class GallicaTask(DefaultTask):
    """
    Base task for Gallica.
    """
    TAG = '20'

    def closest(self):
        return monthly(date=self.date)


class GallicaHarvest(GallicaTask):
    """
    Harvest from default http://oai.bnf.fr/oai2/OAIHandler OAI endpoint.
    """
    format = luigi.Parameter(default='oai_dc')
    date = ClosestDateParameter(default=datetime.date.today())
    endpoint = luigi.Parameter(default='http://oai.bnf.fr/oai2/OAIHandler', significant=False)

    def run(self):
        shellout("""metha-sync -rm -format {format} {endpoint} """, endpoint=self.endpoint, format=self.format)
        output = shellout("""metha-cat -format {format} {endpoint} > {output}""",
                          endpoint=self.endpoint,
                          format=self.format)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='xml'))


class GallicaMARC(GallicaTask):
    """
    Custom convert to MARC.
    """
    date = ClosestDateParameter(default=datetime.date.today())
    format = luigi.Parameter(default='oai_dc')

    def requires(self):
        return GallicaHarvest(date=self.date, format=self.format)

    def run(self):
        output = shellout("python {script} {input} {output}",
                          script=self.assets("20/20_marcbinary.py"),
                          input=self.input().path)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='fincmarc.mrc'))



#!/usr/bin/env python3
# coding: utf-8

import io
import re
import sys

import xmltodict

import marcx
from siskin.mappings import formats
from siskin.utils import check_isbn, check_issn, marc_build_field_008


def get_field(name):

    multivalued_fields = ["dc:identifier", "dc:subject", "dc:language"]
    field = xmlrecord["metadata"]["oai_dc:dc"].get(name, None)
    if not field:
        return ""
    if isinstance(field, list) and name not in multivalued_fields:
        print("Das Feld %s liegt teilweise als Liste vor: " % name)
    return field


inputfilename = "107_input.xml"
outputfilename = "107_output.mrc"

if len(sys.argv) == 3:
    inputfilename, outputfilename = sys.argv[1:]

inputfile = open(inputfilename, "rb")
outputfile = open(outputfilename, "wb")

xmlfile = inputfile.read()
xmlrecords = xmltodict.parse(xmlfile, force_list="dc:identifier")

for xmlrecord in xmlrecords["Records"]["Record"]:

    if not get_field("dc:title"):
        continue

    marcrecord = marcx.Record(force_utf8=True)
    marcrecord.strict = False

    # Formatzuordnung
    #Article</dc:type>
    #Charter</dc:type>
    #Fragment</dc:type>
    #Letter</dc:type>
    #Manuscript</dc:type>
    #Map</dc:type>
    #Monograph</dc:type>
    #Multivolume_work</dc:
    #Periodical</dc:type>
    #Series</dc:type>
    #Text</dc:type>
    #Volume</dc:type>
    format = get_field("dc:type")

    if format == "Article":
        format = "Article"
    elif format == "Manuscript":
        format = "Manuscript"
    elif format == "Periodical":
        format = "Journal"
    elif format == "Multivolume_work":
        format = "Multipart"
    elif format == "Map":
        format = "Map"
    else:
        format = "Book"

    # Leader
    leader = formats[format]["Leader"]
    marcrecord.leader = leader

    # Identifier
    f001 = xmlrecord["header"]["identifier"][0]
    regexp = re.match("oai:digi.ub.uni-heidelberg.de:(\d+)", f001)
    if regexp:
        f001 = regexp.group(1)
        marcrecord.add("001", data="finc-107-" + f001)
    else:
        sys.exit("Der Identifier konnte nicht zerlegt werden: " + f001)

    # Zugangsart
    f007 = formats[format]["e007"]
    marcrecord.add("007", data=f007)

    # Periodizität
    year = get_field("dc:date")
    periodicity = formats[format]["008"]
    language = get_field("dc:language")
    f008 = marc_build_field_008(year, periodicity, language)
    marcrecord.add("008", data=f008)

    # Sprache
    language = get_field("dc:language")
    marcrecord.add("041", a=language)

    # Verfasser
    f100a = get_field("dc:creator")
    marcrecord.add("100", a=f100a)

    # Titel
    f245 = get_field("dc:title")
    marcrecord.add("245", a=f245)

    # Erscheinungsvermerk
    f260c = get_field("dc:date")
    publisher = ["b", "Hochschule Mittweida,", "c", f260c]
    marcrecord.add("260", subfields=publisher)

    # RDA-Inhaltstyp
    f336b = formats[format]["336b"]
    marcrecord.add("336", b=f336b)

    # RDA-Datenträgertyp
    f338b = formats[format]["338b"]
    marcrecord.add("338", b=f338b)

    # Schlagwörter
    f689a = get_field("dc:subject")
    if isinstance(f689a, list):
        for subject in f689a:
            if ";" not in subject:
                marcrecord.add("689", a=subject)
    else:
        f689a = f689a.split(" , ")
        if len(f689a) > 1:
            for subject in f689a:
                if ";" not in subject:
                    marcrecord.add("689", a=subject)

    # Link zu Datensatz und Ressource
    urls = get_field("dc:identifier")
    for url in urls:
        if "http" in url:
            marcrecord.add("856", q="text/html", _3="Link zum Datensatz", u=url)
        if "urn:" in url:
            marcrecord.add("856", q="text/html", _3="Link zur Ressource", u=url)

    # SWB-Inhaltstyp
    f935c = formats[format]["935c"]
    marcrecord.add("935", c=f935c)

    # Ansigelung und Kollektion
    marcrecord.add("980", a=f001, b="107", c="sid-107-col-heidelberg")

    outputfile.write(marcrecord.as_marc())

inputfile.close()
outputfile.close()



#/usr/bin/env python
# coding: utf-8

import sys

import marcx
import pymarc

# def copyfield(srec, sspec, drec, dspec):
#   """
#   Source record and field specification. If this this field exists and is not None, then copy to destination record field.

#   Example:
#         ...
#       copyfield(oldrecord, "520.a", newrecord, "689.b")
#   """
#   pass

# Default input and output.
inputfilename = "143_input.xml"
outputfilename = "143_output.mrc"

if len(sys.argv) == 3:
    inputfilename, outputfilename = sys.argv[1:]

oldrecords = pymarc.marcxml.parse_xml_to_array(inputfilename)
outputfile = open(outputfilename, "wb")

for oldrecord in oldrecords:

    if not oldrecord["001"] or not oldrecord.leader or not oldrecord["856"] or not oldrecord["856"]["u"]:
        continue

    newrecord = marcx.Record(force_utf8=True)

    # Leader
    leader = oldrecord.leader
    newrecord.leader = "     %s  22        4500" % leader[5:8]

    # ID
    f001 = oldrecord["001"].data
    newrecord.add("001", data="finc-143-%s" % f001)

    # 007
    newrecord.add("007", data="vu")

    # 008
    f008 = oldrecord["008"].data
    f008 = f008.replace("\"", " ")
    newrecord.add("008", data=f008)

    # 035
    try:
        f035a = oldrecord["035"]["a"]
        newrecord.add("035", a=f035a)
    except:
        pass

    # 040
    f040a = oldrecord["040"]["a"]
    newrecord.add("040", a=f040a)

    # 1. Schöpfer
    f100a = oldrecord["100"]["a"]
    newrecord.add("100", a=f100a)

    # Haupttitel, Titelzusatz
    f245a = oldrecord["245"]["a"]
    f245b = oldrecord["245"]["b"]
    if f245a or f245b:
        newrecord.add("245", a=f245a, b=f245b)

    # Erscheinungsort, Verlag, Erscheinungsjahr
    f260a = oldrecord["260"]["a"]
    f260b = oldrecord["260"]["b"]
    f260c = oldrecord["260"]["c"]
    publisher = ["a", f260a + " : ", "b", f260b + ", ", "c", f260c]
    newrecord.add("260", subfields=publisher)

    # Umfangsangabe
    f300a = oldrecord["300"]["a"]
    newrecord.add("300", a=f300a)

    # Reihe
    f490a = oldrecord["490"]["a"]
    newrecord.add("490", a=f490a)

    # allgemeine Fußnote
    f500a = oldrecord["500"]["a"]
    newrecord.add("500", a=f500a)

    # Zusammenfassung
    f520a = oldrecord["520"]["a"]
    newrecord.strict = False
    newrecord.add("520", a=f520a)
    newrecord.strict = True

    # Zielgruppe
    try:
        f521a = oldrecord["521"]["a"]
        newrecord.add("521", a=f521a)
    except:
        pass

    # Sprachvermerk
    try:
        f546a = oldrecord["546"]["a"]
        newrecord.add("546", a=f546a)
    except:
        pass

    # Schlagwort
    try:
        f689a = oldrecord["650"]["a"]
        newrecord.add("689", a=f689a)
    except:
        pass

    # URL
    f856u = oldrecord["856"]["u"]
    newrecord.add("856", q="text/html", _3="Link zur Ressource", u=f856u)

    # 935
    newrecord.add("935", b="cofz")
    newrecord.add("935", c="vide")

    # Kollektion
    f980a = f001
    if f490a == "Biology":
        newrecord.add("980", a=f980a, b="143", c="sid-143-col-jovebiology")
    elif f490a == "Medicine":
        newrecord.add("980", a=f980a, b="143", c="sid-143-col-jovemedicine")
    elif f490a == "Neuroscience":
        newrecord.add("980", a=f980a, b="143", c="sid-143-col-joveneuroscience")
    elif f490a == "Immunology and Infection":
        newrecord.add("980", a=f980a, b="143", c="sid-143-col-joveimmunologyandinfection")
    elif f490a == "Bioengineering":
        newrecord.add("980", a=f980a, b="143", c="sid-143-col-jovebioengineering")
    elif f490a == "Behavior":
        newrecord.add("980", a=f980a, b="143", c="sid-143-col-jovebehavior")
    elif f490a == "Biochemistry":
        newrecord.add("980", a=f980a, b="143", c="sid-143-col-jovebiochemistry")
    elif f490a == "Cancer Research":
        newrecord.add("980", a=f980a, b="143", c="sid-143-col-jovecancerresearch")
    elif f490a == "Engineering":
        newrecord.add("980", a=f980a, b="143", c="sid-143-col-joveengineering")
    elif f490a == "Environment":
        newrecord.add("980", a=f980a, b="143", c="sid-143-col-jovedeenvironment")
    elif f490a == "Chemistry":
        newrecord.add("980", a=f980a, b="143", c="sid-143-col-jovechemistry")
    elif f490a == "Developmental Biology":
        newrecord.add("980", a=f980a, b="143", c="sid-143-col-jovedevelopmentalbiology")
    elif f490a == "Genetics":
        newrecord.add("980", a=f980a, b="143", c="sid-143-col-jovegenetics")
    else:
        print("Kollektion %s in Feld 490 unbekannt" % f490a)

    outputfile.write(newrecord.as_marc())

outputfile.close()



# coding: utf-8
# pylint: disable=C0301,E1101

# Copyright 2016 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>
"""
BASE, refs #5994.

* https://www.base-search.net/
* https://api.base-search.net/
* https://en.wikipedia.org/wiki/BASE_(search_engine)
* http://www.dlib.org/dlib/june04/lossau/06lossau.html

Currently (Dec 2018) BASE is provided via SLUB download.

Config
------

[base]

ftp-host = sftp://example.com
ftp-base = /target_data/xyz
ftp-username= admin
ftp-password= s3cr3t
ftp-pattern = *
"""

import datetime

import luigi

from gluish.format import TSV, Gzip
from gluish.intervals import weekly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.common import FTPMirror
from siskin.task import DefaultTask


class BaseTask(DefaultTask):
    """
    Various tasks around base, refs #14947.
    """
    TAG = '126'

    def closest(self):
        return weekly(date=self.date)


class BasePaths(BaseTask):
    """
    Mirror SLUB FTP for base.
    """
    date = ClosestDateParameter(default=datetime.date.today())
    max_retries = luigi.IntParameter(default=10, significant=False)
    timeout = luigi.IntParameter(default=20, significant=False, description='timeout in seconds')

    def requires(self):
        return FTPMirror(host=self.config.get('base', 'ftp-host'),
                         base=self.config.get('base', 'ftp-base'),
                         username=self.config.get('base', 'ftp-username'),
                         password=self.config.get('base', 'ftp-password'),
                         pattern=self.config.get('base', 'ftp-pattern'),
                         max_retries=self.max_retries,
                         timeout=self.timeout)

    def run(self):
        self.input().move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(), format=TSV)



#!/usr/bin/env python
# coding: utf-8

import io
import logging
import re
import sys
from builtins import *
from io import BytesIO, StringIO

import marcx
import pymarc
from siskin.utils import marc_clean_record, xmlstream

inputfilename = "39_input.xml"
outputfilename = "39_output.mrc"
filterfilename = "39_issn_filter"

if len(sys.argv) >= 4:
    inputfilename, outputfilename, filterfilename = sys.argv[1:4]

outputfile = io.open(outputfilename, "wb")
filterfile = io.open(filterfilename, "r")

issn_list = filterfile.readlines()
issn_list = [issn.rstrip("\n") for issn in issn_list]

for record in xmlstream(inputfilename, "record"):
    record = BytesIO(record)
    record = pymarc.marcxml.parse_xml_to_array(record)
    record = record[0]

    record = marcx.Record.from_record(record)
    record.force_utf8 = True
    record.strict = False

    # prüfen, ob Titel vorhanden ist
    if not record["245"]:
        continue

    # Leader
    record.leader = "     " + record.leader[5:]

    # Identifikator
    f001 = record["001"].data
    record.remove_fields("001")
    f001 = f001.replace("-", "").replace("_", "")
    record.add("001", data="finc-39-%s" % f001)

    # Kollektion und Ansigelung
    try:
        f022a = record["022"]["a"]
    except:
        f022a = ""

    descriptions = list(record.itervalues('520.a'))
    record.remove_fields("520")

    # Ugly, but helps to emit valid binary MARC.
    descriptions = [d[:8000] for d in descriptions]
    total = sum((len(s) for s in descriptions))
    if total > 9000:
        logging.warn('%s: sum of descriptions exceeds (%d) limit, keeping 1 of %d fields', f001, total,
                     len(descriptions))
        descriptions = descriptions[:1]
    for description in descriptions:
        record.add("520", a=description)

    try:
        f760x = record["760"]["x"]
    except:
        f760x = ""

    try:
        f787x = record["787"]["x"]
    except:
        f787x = ""

    if f022a in issn_list or f760x in issn_list or f787x in issn_list:
        collections = ["a", f001, "b", "39", "c", u"sid-39-col-persee", "c", u"sid-39-col-perseeadlr"]
    else:
        collections = ["a", f001, "b", "39", "c", u"sid-39-col-persee"]

    record.add("980", subfields=collections)

    marc_clean_record(record)
    outputfile.write(record.as_marc())

outputfile.close()
filterfile.close()



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,R0904,E1103,C0301
# Copyright 2015 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Tracy Hoffmann, <tracy.hoffmann@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>
"""
NRW music tasks. Deprecated, refs #9014.

Config
------

[nrw]

url56 = url_56
url57 = url_57
url58 = url_58

"""

import datetime
import email.utils as eut

import luigi
import requests

from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.task import DefaultTask


class NRWTask(DefaultTask):
    """ Base task for NRW music (sid 56,57,58) """
    TAG = 'nrw'

    def closest(self):
        """
        Use the HTTP Last-Modified header to determine the most recent date.

        If we cannot determine the date, we fail (maybe fallback to some weekly value instead).
        """
        if not hasattr(self, 'sid'):
            raise AttributeError('assumed task has a parameter sid, but it does not')

        url = self.config.get('nrw', 'url%s' % self.sid)

        resp = requests.head(url)
        if resp.status_code != 200:
            raise RuntimeError('%s on %s' % (resp.status_code, self.url))

        value = resp.headers.get('Last-Modified')
        if value is None:
            raise RuntimeError('HTTP Last-Modified header not found')

        parsed_date = eut.parsedate(value)
        if parsed_date is None:
            raise RuntimeError('could not parse Last-Modifier header')

        last_modified_date = datetime.date(*parsed_date[:3])

        return last_modified_date


class NRWHarvest(NRWTask):
    """
    Download dump and unzip
    """
    date = ClosestDateParameter(default=datetime.date.today())
    sid = luigi.Parameter(default='56', description='source id')

    def run(self):
        url = self.config.get('nrw', 'url%s' % self.sid)
        output = shellout("""curl --fail {url} | tar -xOz > {output}""", url=url)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='xml'))


class NRWTransformation(NRWTask):
    """
    Transform data for finc
    """
    date = ClosestDateParameter(default=datetime.date.today())
    sid = luigi.Parameter(default='56', description='source id')

    def requires(self):
        return NRWHarvest(date=self.date, sid=self.sid)

    def run(self):
        output = shellout("""flux.sh {flux} in={input} sid={sid} > {output}""",
                          flux=self.assets("56_57_58/nrw_music.flux"),
                          input=self.input().path,
                          sid=self.sid)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='fincmarc.xml'))



# coding: utf-8
# pylint: disable=C0301,E1101

# Copyright 2017 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>
"""
ZVDD, refs #6610.
"""

import datetime

import luigi

from gluish.format import Gzip
from gluish.intervals import quarterly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.task import DefaultTask


class ZVDDTask(DefaultTask):
    """
    Base task for ZVDD, refs #6610.
    """
    TAG = '93'

    def closest(self):
        return quarterly(self.date)


class ZVDDHarvest(ZVDDTask):
    """
    Harvest ZVDD endpoint. Default oai_dc does not contain too much
    information, harvesting METS with metha yielded errors. Use oaicrawl
    (https://git.io/v5yuQ) to harvest endpoint occasionally. One complete
    download might take several days. Data set is created in a best effort
    style (errors ignored).

    Estimated number of records (Fall 2017): ~502638.

    This harvest may take a day or two.
    """
    date = ClosestDateParameter(default=datetime.date.today())
    url = luigi.Parameter(default="http://zvdd.de/oai2/", significant=False)
    prefix = luigi.Parameter(default="mets", significant=False)

    def run(self):
        """
        ---- 8< ----

        Usage of oaicrawl:
        -b      create best effort data set
        -e duration
                max elapsed time (default 10s)
        -f string
                format (default "oai_dc")
        -retry int
                max number of retries (default 3)
        -verbose
                more logging
        -version
                show version
        -w int
                number of parallel connections (default 16)
        """
        output = shellout("""oaicrawl -w 12 -retry 5 -e 15s -f {prefix} -b -verbose "{url}" | pigz -c > {output}""",
                          prefix=self.prefix,
                          url=self.url)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='data.gz'), format=Gzip)


class ZVDDIntermediateSchema(ZVDDTask):
    """
    Experimental conversion to intermediate format.
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return ZVDDHarvest(date=self.date)

    def run(self):
        output = shellout("unpigz -c {input} | span-import -i zvdd-mets | pigz -c > {output}", input=self.input().path)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj.gz'), format=Gzip)



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,R0904,E1103,C0301

# Copyright 2017 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>
#
"""
Wiso aux tasks.

Config
------

[redmine]
baseurl = https://example.projects.de
apikey = d24411122381317432647264876234

"""

import datetime
import os
import tempfile

import luigi

from gluish.utils import shellout
from siskin.common import RedmineDownloadAttachments
from siskin.task import DefaultTask


class WisoTask(DefaultTask):
    """
    An auxiliary namespace for WISO related tasks, refs #12301.
    """
    TAG = 'wiso'


class Wiso2018Files(WisoTask):
    """
    Combine into single file.

    >>> import pandas as pd
    >>> df = pd.read_csv("output.csv", sep=";", encoding='latin1')
    >>> df.columns
    Index(['wiso-net_FZS_Psychologie_2018.csv', 'Zeitschrift', 'Regionalausgaben',
        'Sprache', 'Verlag', 'ISSN', '2. ISSN', 'Beginn Volltextarchiv (Datum)',
        'Ende Volltextarchiv (Datum)', 'Beginn Volltextarchiv (Heft)',
        'Beginn Volltextarchiv (Jahr)', 'Ende Volltextarchiv (Heft)',
        'Ende Volltextarchiv (Jahr)', 'Verzug', 'URL Inhaltsverzeichnis',
        'URL Artikelebene', 'URL Publikation', 'Schlagworte', 'Bemerkungen',
        'Aktualisierung', 'DB-Kürzel', 'Produktsigel'],
        dtype='object')

    """

    def requires(self):
        return RedmineDownloadAttachments(issue="12301")

    def run(self):
        with self.input().open() as handle:
            _, combined = tempfile.mkstemp(prefix='siskin-')
            for row in handle.iter_tsv(cols=('path', )):
                # Read CSV file and prepend the basename of the file to the line.
                filename = os.path.basename(row.path)
                shellout(""" awk '{{print "{filename};"$0}}' < {input} >> {output}""",
                         filename=filename,
                         input=row.path,
                         output=combined)

        luigi.LocalTarget(combined).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='csv'))



from siskin.openurl import openurl_parameters_from_intermediateschema


def test_openurl_from_intermediateschema():
    cases = (
        ('empty doc', {}, {}),
        (
            'title only',
            {
                'rft.atitle': 'empty doc'
            },
            {
                'ctx_enc': 'info:ofi/enc:UTF-8',
                'ctx_ver': 'Z39.88-2004',
                'rfr_id': 'info:sid/www.ub.uni-leipzig.de:generator',
                'rft.atitle': 'empty doc',
                'url_ver': 'Z39.88-2004',
            },
        ),
        (
            'title and date',
            {
                'rft.atitle': 'title and date',
                'rft.date': '2018-10-10',
            },
            {
                'ctx_enc': 'info:ofi/enc:UTF-8',
                'ctx_ver': 'Z39.88-2004',
                'rfr_id': 'info:sid/www.ub.uni-leipzig.de:generator',
                'rft.date': '2018-10-10',
                'rft.atitle': 'title and date',
                'url_ver': 'Z39.88-2004',
            },
        ),
        (
            'title and date, language',
            {
                'languages': ['eng', 'fra'],
                'rft.atitle': 'title and date, language',
                'rft.date': '2018-10-10',
            },
            {
                'ctx_enc': 'info:ofi/enc:UTF-8',
                'ctx_ver': 'Z39.88-2004',
                'rfr_id': 'info:sid/www.ub.uni-leipzig.de:generator',
                'rft.date': '2018-10-10',
                'rft.language': 'eng',
                'rft.atitle': 'title and date, language',
                'url_ver': 'Z39.88-2004',
            },
        ),
        (
            'title and date, language, book',
            {
                'languages': ['eng', 'fra'],
                'rft.atitle': 'Hello',
                'rft.date': '2018-10-10',
                'rft.genre': 'book',
            },
            {
                'ctx_enc': 'info:ofi/enc:UTF-8',
                'ctx_ver': 'Z39.88-2004',
                'rfr_id': 'info:sid/www.ub.uni-leipzig.de:generator',
                'rft.atitle': 'Hello',
                'rft.btitle': 'Hello',
                'rft.date': '2018-10-10',
                'rft.genre': 'book',
                'rft.language': 'eng',
                'rft_val_fmt': 'info:ofi/fmt:kev:mtx:book',
                'url_ver': 'Z39.88-2004',
            },
        ),
        (
            'crossref-1',
            {
                "finc.format":
                "ElectronicArticle",
                "finc.mega_collection": ["Springer Science + Business Media (CrossRef)"],
                "finc.id":
                "ai-49-aHR0cDovL2R4LmRvaS5vcmcvMTAuMTAxNi9qLm51cnguMjAwNi4wNS4wMjU",
                "finc.source_id":
                "49",
                "ris.type":
                "EJOUR",
                "rft.atitle":
                "An Analysis of Correlations Among 4 Outcome Scales Employed in Clinical Trials of Patients With Major Depressive Disorder",
                "rft.epage":
                "412",
                "rft.genre":
                "article",
                "rft.issn": ["1545-5343"],
                "rft.issue":
                "3",
                "rft.jtitle":
                "NeuroRX",
                "rft.tpages":
                "2",
                "rft.pages":
                "411-412",
                "rft.pub": ["Springer Science + Business Media"],
                "rft.date":
                "2006-07-01",
                "x.date":
                "2006-07-01T00:00:00Z",
                "rft.spage":
                "411",
                "rft.volume":
                "3",
                "authors": [{
                    "rft.aulast": "JIANG",
                    "rft.aufirst": "Q"
                }, {
                    "rft.aulast": "AHMED",
                    "rft.aufirst": "S"
                }, {
                    "rft.aulast": "PEDERSEN",
                    "rft.aufirst": "R"
                }, {
                    "rft.aulast": "MUSGNUNG",
                    "rft.aufirst": "J"
                }, {
                    "rft.aulast": "ENTSUAH",
                    "rft.aufirst": "R"
                }],
                "doi":
                "10.1016/j.nurx.2006.05.025",
                "languages": ["eng"],
                "url": ["http://dx.doi.org/10.1016/j.nurx.2006.05.025"],
                "version":
                "0.9",
                "x.subjects": ["Pharmacology (medical)"],
                "x.type":
                "journal-article"
            },
            {
                'ctx_enc': 'info:ofi/enc:UTF-8',
                'ctx_ver': 'Z39.88-2004',
                'rfr_id': 'info:sid/www.ub.uni-leipzig.de:generator',
                'rft.atitle':
                'An Analysis of Correlations Among 4 Outcome Scales Employed in Clinical Trials of Patients With Major Depressive Disorder',
                'rft.aufirst': 'Q',
                'rft.aulast': 'JIANG',
                'rft.date': '2006-07-01',
                'rft.epage': '412',
                'rft.genre': 'article',
                'rft.issn': '1545-5343',
                'rft.issue': '3',
                'rft.jtitle': 'NeuroRX',
                'rft.language': 'eng',
                'rft.pages': '411-412',
                'rft.spage': '411',
                'rft.volume': '3',
                'rft_id': 'info:doi/10.1016/j.nurx.2006.05.025',
                'url_ver': 'Z39.88-2004',
            },
        ),
    )

    for _, doc, want in cases:
        result = openurl_parameters_from_intermediateschema(doc)
        assert result == want



# coding: utf-8
# pylint: disable=C0301,E1101

# Copyright 2015 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>
"""
datacite.org

> Locate, identify, and cite research data with the leading global provider of DOIs for research data.

Config:

[core]

metha-dir = /path/to/dir

"""

import datetime

import luigi

from gluish.common import Executable
from gluish.intervals import monthly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.task import DefaultTask


class DataciteTask(DefaultTask):
    """
    Base task.
    """

    def closest(self):
        return monthly(date=self.date)


class DataciteCombine(DataciteTask):
    """
    Single file dump.
    """
    date = ClosestDateParameter(default=datetime.date.today())
    url = luigi.Parameter(default="http://oai.datacite.org/oai", significant=False)
    prefix = luigi.Parameter(default="oai_dc", significant=False)

    def requires(self):
        return Executable(name='metha-sync', message='https://github.com/miku/metha')

    def run(self):
        shellout("METHA_DIR={dir} metha-sync -format {prefix} {url}",
                 prefix=self.prefix,
                 url=self.url,
                 dir=self.config.get('core', 'metha-dir'))
        output = shellout("METHA_DIR={dir} metha-cat -root Records -format {prefix} {url} | pigz -c > {output}",
                          prefix=self.prefix,
                          url=self.url,
                          dir=self.config.get('core', 'metha-dir'))
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext="xml.gz"))


class DataciteIntermediateSchema(DataciteTask):
    """
    Experimental.
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return DataciteCombine(date=self.date)

    def run(self):
        mapdir = 'file:///%s' % self.assets("maps/")
        output = shellout("""flux.sh {flux} in={input} MAP_DIR={mapdir} | pigz -c > {output}""",
                          flux=self.assets("datacite/flux.flux"),
                          mapdir=mapdir,
                          input=self.input().path)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj.gz'))


class DataciteExport(DataciteTask):
    """
    Export to various formats
    """
    date = ClosestDateParameter(default=datetime.date.today())
    format = luigi.Parameter(default='solr5vu3', description='export format')

    def requires(self):
        return DataciteIntermediateSchema(date=self.date)

    def run(self):
        output = shellout("span-export -o {format} <(unpigz -c {input}) | pigz -c > {output}",
                          format=self.format,
                          input=self.input().path)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        extensions = {
            'solr5vu3': 'ldj.gz',
            'formeta': 'form.gz',
        }
        return luigi.LocalTarget(path=self.path(ext=extensions.get(self.format, 'gz')))



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,E1103,C0301,C0103,W0614,W0401,E0202

# Copyright 2019 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>
"""
Slight MABXML abstraction layer, using xmltodict under the hood. Usable up to
file sizes of 50M. TODO: For larger files, rewrite this with streaming. MAB is
being phased out since 2013.

MAB specification: https://www.dnb.de/DE/Standardisierung/Formate/MAB/mab_node.html

Example MABXML files: https://git.io/fhFHr.

    from siskin.mab import MabXMLFile

    mabfile = MabXMLFile("fixtures/mab0.xml")

    for record in mabfile:

        # Return the first value or None.
        title = record.field("331")
        if not title:
            raise ValueError("record has not title")

        # Return multiple values (or empty list).
        for isbn in record.fields("540"):
            print(isbn)

"""

import io
import os

import six
import xmltodict


class MabRecord(object):
    """
    A single MAB record.
    """

    def __init__(self, dd):
        """
        Initialize with the default dictionary created by xmltodict for a
        single datensatz element.
        """
        self.dd = dd

    def status(self):
        return self.dd.get("@status")

    def typ(self):
        return self.dd.get("@typ")

    def mabVersion(self):
        return self.dd.get("@mabVersion")

    def feld(self, number, code=None, alt=None):
        """
        Shortcut, since MAB is a German format.
        """
        return self.field(number, code=code, alt=None)

    def felder(self, number, code=None):
        """
        Shortcut, since MAB is a German format.
        """
        return self.fields(number, code=code)

    def field(self, number, code=None, alt=None):
        """
        Returns the value of the first (only) field matching number or None.
        """
        for f in self.dd.get("feld", []):
            if f.get("@nr") == number:
                if code:
                    for sf in f.get("uf", []):
                        if sf.get("@code") == code:
                            return sf.get("#text", alt)
                else:
                    return f.get("#text", alt)
        return alt

    def fields(self, number, code=None):
        """
        Returns the values of the all fields matching number and possibly
        subfield or empty list.
        """
        result = []

        for f in self.dd.get("feld", []):
            if f.get("@nr") != number:
                continue
            if code:
                for sf in f.get("uf", []):
                    if sf.get("@code") != code:
                        continue
                    if sf.get("#text"):
                        result.append(sf.get("#text"))
            else:
                if f.get("#text"):
                    result.append(f.get("#text"))

        return result

    def size(self):
        return len([_ for _ in self.dd.get("feld", [])])

    def __str__(self):
        return '<MabXMLRecord status=%s, typ=%s, with %s fields>' % (self.status(), self.typ(), self.size())

    def __repr__(self):
        return self.__str__()


class MabXMLFile(object):
    """
    Encapsulate MAB XML methods.
    """

    def __init__(self, data, replace=None, encoding='utf-8'):
        """
        Eagerly parse the XML from filename, filelike or string into a
        xmltodict defaultdict. Basic checks for MAB like XML.

        TODO(miku): Use streaming mode.

        The replace optional argument should be a list of desired substitutions
        in the content, e.g.  (("¬", ""),) to be applied before parsing.
        """
        if replace and len(replace) == 2:
            replace = (replace, )
        if isinstance(data, six.string_types):
            if os.path.exists(data):
                # Assume it is a file.
                with io.open(data, encoding=encoding) as handle:
                    content = handle.read()
            else:
                # Assume direct data.
                content = data
        else:
            # Assume something that can be read from.
            content = data.read()

        if replace:
            for value, replacement in replace:
                content = content.replace(value, replacement)

        self.dd = xmltodict.parse(content, force_list=('datensatz', 'feld', 'uf'))
        if not "datei" in self.dd:
            raise ValueError("datei tag not found or empty file without fields")

        # It there are no fields, we get an error.
        if self.dd["datei"] is None:
            self.records = []
        else:
            self.records = self.dd["datei"].get("datensatz", [])

    def __iter__(self):
        return self

    def __next__(self):
        """
        Temporary attribute for iteration.
        """
        if not hasattr(self, "_it"):
            self._it = iter(self.records)
        try:
            # http://diveinto.org/python3/porting-code-to-python-3-with-2to3.html#next
            if hasattr(self._it, "next"):
                return MabRecord(self._it.next())
            return MabRecord(next(self._it))
        except StopIteration:
            delattr(self, "_it")
            raise

    def next(self):
        """
        Python 2 compatibility.
        """
        return self.__next__()

    def __str__(self):
        return '<MabXMLFile>'

    def __repr__(self):
        return self.__str__()



#!/usr/bin/env python
# coding: utf-8
#
# Copyright 2019 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Robert Schenk, <robert.schenk@uni-leipzig.de>
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""

Automated content verification of all sources in the live solr
Ticket: #15656

"""

from __future__ import print_function

import argparse
import io
import os
import re
import smtplib
import sqlite3
import sys
import tempfile
import time
from six.moves.urllib.parse import urlencode
from sqlite3 import Error

import requests

# The current database schema.
create_schema = """
    CREATE TABLE
        source
            (source INT PRIMARY KEY NOT NULL);

    CREATE TABLE
        institution
            (institution VARCHAR(30) PRIMARY KEY NOT NULL);

    CREATE TABLE
        sourcebyinstitution
            (sourcebyinstitution VARCHAR(30) PRIMARY KEY NOT NULL);

    CREATE TABLE
        history
            (date DEFAULT CURRENT_TIMESTAMP,
            sourcebyinstitution VARCHAR(30) NOT NULL,
            titles INT NOT NULL);
"""

# XXX: Encapsulate this better, to get rid of globals.
smtp_server = "mail.example.com"
smtp_port = 465
smtp_sender = "noreply@example.com"
smtp_name = "username"
smtp_password = "password"
recipients = "a@example.com,b@example.com"


def send_message(message):
    """
    Send e-mail to preconfigured recipients.
    """
    server = smtplib.SMTP(smtp_server, smtp_port)
    server.starttls()
    server.login(smtp_name, smtp_password)
    message = "Subject: SolrCheckup Warnung!\n" + message
    server.sendmail(smtp_sender, recipients, message)
    server.quit()


def create_connection_and_set_cursor(database):
    """
    Creates a database connection to a SQLite database and returns a cursor
    """
    try:
        conn = sqlite3.connect(database)
    except Error as e:
        print(e)
        sys.exit("No database connection could be established.")
    cursor = conn.cursor()
    return (conn, cursor)


def get_solr_result(index, params):
    """
    Takes a Solr index and a dict of parameters and returns a result object.
    Index should be hostport or ip:port, like 10.1.1.1:8085.
    """
    params = urlencode(params)
    result = requests.get("http://%s/solr/biblio/select?%s" % (index, params))
    return result.json()


def get_all_current_sources(finc, ai):
    """
    Get all current sources from Solr in both finc main index and ai.
    """
    params = {
        "facet": "true",
        "facet.field": "source_id",
        "facet.mincount": 3, # because of these cases ["", 2, "\" \"", 1]
        "q": "!source_id:error",
        "rows": 0,
        "wt": "json",
    }

    result = get_solr_result(finc, params)
    finc_sources = result["facet_counts"]["facet_fields"]["source_id"]
    finc_sources = set([int(sid) for sid in finc_sources[::2]])

    result = get_solr_result(ai, params)
    ai_sources = result["facet_counts"]["facet_fields"]["source_id"]
    ai_sources = set([int(sid) for sid in ai_sources[::2]])

    shared = finc_sources.intersection(ai_sources)
    if len(shared) > 0:
        ssid = [str(sid) for sid in shared]
        message = "Die folgenden Quellen befinden sich sowohl im finc-main-Index als auch im AI: {}".format(", ".join(ssid))
        send_message(message)

    return finc_sources.union(ai_sources)


def get_all_old_sources(conn, sqlite):
    """
    Get all old sources from the Database.
    """
    query = """
        SELECT
            source
        FROM
            source
        GROUP BY
            source
    """

    sqlite.execute(query)
    old_sources = []

    for record in sqlite:
        old_source = record[0]
        old_sources.append(old_source)

    return old_sources


def update_sources(conn, sqlite, finc, k10plus, ai):
    """
    Update the source table.
    """
    current_sources = get_all_current_sources(finc, ai)
    old_sources = get_all_old_sources(conn, sqlite)

    # Check if the source table is allready filled and this is not the first checkup
    if len(old_sources) > 100:
        source_table_is_filled = True
    else:
        source_table_is_filled = False

    for old_source in old_sources:
        if source_table_is_filled and old_source not in current_sources:
            message = "Die SID %s ist im aktuellen Import nicht mehr vorhanden.\nWenn dies beabsichtigt ist, bitte die SID aus der Datenbank loeschen." % old_source
            send_message(message)

    for current_source in current_sources:
        if current_source not in old_sources:
            print("The source %s is new in Solr." % current_source)
            sql = "INSERT INTO source (source) VALUES (%s)" % current_source
            sqlite.execute(sql)
            conn.commit()


def get_all_current_institutions(finc, ai):
    """
    Get all current institutions from Solr.
    """
    current_institutions = []

    params = {
        "facet": "true",
        "facet.field": "institution",
        "facet.mincount": 3, # because of these cases ["", 2, "\" \"", 1]
        "q": "!source_id:error",
        "rows": 0,
        "wt": "json",
    }

    # check finc main index
    result = get_solr_result(finc, params)
    institutions = result["facet_counts"]["facet_fields"]["institution"]
    for institution in institutions[::2]:
        current_institutions.append(institution)

    # check ai
    result = get_solr_result(ai, params)
    institutions = result["facet_counts"]["facet_fields"]["institution"]
    for institution in institutions[::2]:
        if institution in current_institutions:
            continue
        current_institutions.append(institution)

    return current_institutions


def get_all_old_institutions(conn, sqlite):
    """
    Get all old institutions from the SQLite database.
    """
    query = """
        SELECT
            institution
        FROM
            institution
        GROUP BY
            institution
    """

    sqlite.execute(query)
    old_institutions = []

    for record in sqlite:
        old_institution = record[0]
        old_institutions.append(old_institution)

    return old_institutions


def get_all_old_sourcebyinstitutions(conn, sqlite):
    """
    Get all old sourcebyinstitution from the SQLite database.
    """
    query = """
        SELECT
            sourcebyinstitution
        FROM
            sourcebyinstitution
        GROUP BY
            sourcebyinstitution
    """

    sqlite.execute(query)
    old_sourcebyinstitutions = []

    for record in sqlite:
        old_sourcebyinstitution = record[0]
        old_sourcebyinstitutions.append(old_sourcebyinstitution)

    return old_sourcebyinstitutions


def get_old_sourcebyinstitution_number(conn, sqlite, sourcebyinstitution):
    """
    Get all the old sourcebyinstitution number from the SQLite database.
    """
    query = """
        SELECT
            titles
        FROM
            history
        WHERE
            sourcebyinstitution = "%s"
        ORDER BY
            titles DESC
        LIMIT 1
    """ % sourcebyinstitution

    sqlite.execute(query)
    for record in sqlite:
        old_sourcebyinstitution_number = record[0]
        return old_sourcebyinstitution_number


def update_institutions(conn, sqlite, finc, k10plus, ai):
    """
    Update the institution table.
    """
    current_institutions = get_all_current_institutions(finc, ai)
    old_institutions = get_all_old_institutions(conn, sqlite)

    # Check if the institution table is allready filled and this is not the first checkup
    institution_table_is_filled = len(old_institutions) > 5

    for old_institution in old_institutions:
        if institution_table_is_filled and old_institution not in current_institutions:
            message = "Die ISIL %s ist im aktuellen Import nicht mehr vorhanden.\nWenn dies beabsichtigt ist, bitte die Institution aus der Datenbank loeschen." % old_institution
            send_message(message)

    for current_institution in current_institutions:
        if current_institution == " " or '"' in current_institution:
                continue
        if current_institution not in old_institutions:
            print("The institution %s is new in Solr." % current_institution)
            sql = "INSERT INTO institution (institution) VALUES ('%s')" % current_institution
            sqlite.execute(sql)
            conn.commit()


def update_history_and_sourcebyinstitution(conn, sqlite, finc, k10plus, ai):
    """
    Get all current sources and title numbers from Solr and log them into database.
    """
    current_sources = get_all_current_sources(finc, ai)
    current_institutions = get_all_current_institutions(finc, ai)
    old_sourcebyinstitutions = get_all_old_sourcebyinstitutions(conn, sqlite)
    current_sourcebyinstitutions = []

    for source in current_sources:

        for institution in current_institutions:

            if not institution or institution == " " or '"' in institution:
                continue

            sourcebyinstitution = "SID " + str(source) + " (" + institution + ")"
            current_sourcebyinstitutions.append(sourcebyinstitution)

            params = {
                "q": 'source_id:%s AND institution:"%s"' % (source, institution),
                "rows": 0,
                "wt": "json"
            }

            # check finc main
            result = get_solr_result(finc, params)
            number = result["response"]["numFound"]
            if number != 0:
                sql = 'INSERT INTO history (sourcebyinstitution, titles) VALUES ("%s", %s)' % (sourcebyinstitution, number)
                sqlite.execute(sql)
                conn.commit()
            else:
                # check ai
                result = get_solr_result(ai, params)
                number = result["response"]["numFound"]
                if number != 0:
                    # TODO: escape via sqlite
                    sql = 'INSERT INTO history (sourcebyinstitution, titles) VALUES ("%s", %s)' % (sourcebyinstitution, number)
                    sqlite.execute(sql)
                    conn.commit()

            if sourcebyinstitution not in old_sourcebyinstitutions:
                print("The %s is now connected to SID %s." % (institution, source))
                sql = "INSERT INTO sourcebyinstitution (sourcebyinstitution) VALUES ('%s')" % sourcebyinstitution
                sqlite.execute(sql)
                conn.commit()

            if number != 0:
                old_sourcebyinstitution_number = get_old_sourcebyinstitution_number(conn, sqlite, sourcebyinstitution)
                if number < old_sourcebyinstitution_number:
                    message = "Die Anzahl der Titel hat sich bei %s gegenueber einem frueheren Import verringert." % (sourcebyinstitution)
                    send_message(message)

            # requests.exceptions.ConnectionError: HTTPConnectionPool(XXXXXX): Max retries exceeded
            time.sleep(0.25)

    for old_sourcebyinstitution in old_sourcebyinstitutions:
        if old_sourcebyinstitution not in current_sourcebyinstitutions:
            message = "Die %s ist nicht laenger für die SID %s angesigelt." % (institution, source)
            send_message(message)

# Parse keyword arguments
parser = argparse.ArgumentParser()
parser.add_argument("-v",
                    action="version",
                    help="show version",
                    version="0.0.1")
parser.add_argument("-d",
                    dest="database",
                    help="path to database",
                    metavar="database")
parser.add_argument("-y",
                    dest="yaml",
                    help="link to review.yaml or another yaml template",
                    metavar="yaml")
parser.add_argument("-t",
                    dest="token",
                    help="private token for GitLab",
                    metavar="token")
parser.add_argument("-f",
                    dest="finc",
                    help="url of the finc main index",
                    metavar="finc")
parser.add_argument("-k",
                    dest="k10plus",
                    help="url of the k10plus index",
                    metavar="k10plus")
parser.add_argument("-a",
                    dest="ai",
                    help="url of the ai index",
                    metavar="ai")
parser.add_argument("-n", "--smtp-name",
                    dest="smtp_name",
                    help="the login name fpr the email account",
                    metavar="smtp-name")
parser.add_argument("-p", "--smtp-password",
                    dest="smtp_password",
                    help="the password of the email account",
                    metavar="smtp_password")
parser.add_argument("--smtp-server",
                    dest="smtp_server",
                    help="SMTP server",
                    metavar="smtp_server")
parser.add_argument("--smtp-port",
                    dest="smtp_port",
                    help="SMTP port",
                    metavar="smtp_port",
                    default=465)
parser.add_argument("--smtp-sender",
                    dest="smtp_sender",
                    help="SMTP from address",
                    metavar="smtp_sender",
                    default="noreply@example.com")
parser.add_argument("--recipients",
                    dest="recipients",
                    help="recipients for alert messages, comma separated",
                    metavar="recipients")

args = parser.parse_args()

# XXX: Reduce use of globals.
smtp_server = args.smtp_server
smtp_port = args.smtp_port
smtp_server = args.smtp_server
smtp_sender = args.smtp_sender
smtp_name = args.smtp_name
smtp_password = args.smtp_password
recipients = args.recipients


# Set default path if no database was specified
database = args.database
if not database:
    database = os.path.join(tempfile.gettempdir(), "solrcheckup.sqlite")

# Exit when using yaml template without private token
yaml = args.yaml
token = args.token
if yaml and not token:
    sys.exit("Keyword argument for private token needed when using yaml template.")

# Ensure that all three indicies are specified
finc = args.finc
k10plus = args.k10plus
ai = args.ai
if not finc or not k10plus or not ai:
    sys.exit("Three keyword arguments needed for finc, k10plus and ai index.")


# Check if database already exists, otherwise create new one
if not os.path.isfile(database):
    conn, sqlite = create_connection_and_set_cursor(database)
    sqlite.executescript(create_schema)
else:
    conn, sqlite = create_connection_and_set_cursor(database)

# 1. Step: Update the source table
update_sources(conn, sqlite, finc, k10plus, ai)

# 2. Step: Update the institution table
update_institutions(conn, sqlite, finc, k10plus, ai)

# 3. Step: Get the number of titles for each SID and log them to database
update_history_and_sourcebyinstitution(conn, sqlite, finc, k10plus, ai)

sqlite.close()



#!/usr/bin/env python3
# coding: utf-8

import io
import re
import sys

import xmltodict

import marcx

formatmaps = {
    '': {
        'leader': '',
        '007': '',
        '008': '',
        '935b': '',
        '935c': ''
    },
    '': {
        'leader': '',
        '007': '',
        '008': '',
        '935b': '',
        '935c': ''
    },
    '': {
        'leader': '',
        '007': '',
        '008': '',
        '935b': '',
        '935c': ''
    },
    '': {
        'leader': '',
        '007': '',
        '008': '',
        '935b': '',
        '935c': ''
    },
    '': {
        'leader': '',
        '007': '',
        '008': '',
        '935b': '',
        '935c': ''
    },
    '': {
        'leader': '',
        '007': '',
        '008': '',
        '935b': '',
        '935c': ''
    },
    '': {
        'leader': '',
        '007': '',
        '008': '',
        '935b': '',
        '935c': ''
    }
}

inputfilename = "170_input.xml"
outputfilename = "170_output.mrc"

if len(sys.argv) == 3:
    inputfilename, outputfilename = sys.argv[1:]

inputfile = open(inputfilename, "rb")
outputfile = open(outputfilename, "wb")

xmlfile = inputfile.read()
xmlrecords = xmltodict.parse(xmlfile)

for xmlrecord in xmlrecords["Records"]["Record"]:

    if not xmlrecord["metadata"]["oai_dc:dc"].get("dc:title"):
        continue

    marcrecord = marcx.Record(force_utf8=True)

    # Leader
    format = xmlrecord["metadata"]["oai_dc:dc"]["dc:type"]
    if format[0] == "article":
        marcrecord.leader = "     nab  22        4500"
    else:
        marcrecord.leader = "     cam  22        4500"

    # Identifier
    f001 = xmlrecord["header"]["identifier"]
    regexp = re.match("oai:mediarep.org:doc/(\d+)", f001)
    if regexp:
        f001 = regexp.group(1)
        marcrecord.add("001", data="finc-170-" + f001)
    else:
        print(u"Der Identifier konnte nicht zerlegt werden: " + f001)

    # 007
    marcrecord.add("007", data="cr")

    # ISBN
    if xmlrecord["metadata"]["oai_dc:dc"].get("dc:identifier"):
        identifiers = xmlrecord["metadata"]["oai_dc:dc"]["dc:identifier"]
        if isinstance(identifiers, list):
            for identifier in identifiers:
                if re.search("([0-9xX-]{10,17})", identifier):
                    marcrecord.add("020", a=identifier)
                    break

    # ISSN
    if xmlrecord["metadata"]["oai_dc:dc"].get("dc:identifier"):
        identifiers = xmlrecord["metadata"]["oai_dc:dc"]["dc:identifier"]
        if isinstance(identifiers, list):
            for identifier in identifiers:
                identifier = identifier.replace("issn:", "")
                if re.search("\d\d\d\d-\d\d\d\d$", identifier):
                    marcrecord.add("022", a=identifier)
                    break

    # Sprache
    language = xmlrecord["metadata"]["oai_dc:dc"]["dc:language"]
    if language == "de" or language == "deu":
        language = "ger"
    marcrecord.add("008", data="130227uu20uuuuuuxx uuup%s  c" % language)
    marcrecord.add("041", a=language)

    # Fachgebiet
    if xmlrecord["metadata"]["oai_dc:dc"].get("dc:subject"):
        subjects = xmlrecord["metadata"]["oai_dc:dc"]["dc:subject"]
        if isinstance(subjects, list):
            for subject in subjects:
                if re.search("\d\d\d", subject):
                    marcrecord.add("082", a=subject, _2="ddc")
                    break
        else:
            if re.search("\d\d\d", subjects):
                marcrecord.add("084", a=subjects, _2="ddc")

    # 1. Urheber
    if xmlrecord["metadata"]["oai_dc:dc"].get("dc:creator"):
        creator = xmlrecord["metadata"]["oai_dc:dc"]["dc:creator"]
        if isinstance(creator, list):
            marcrecord.add("100", a=creator[0])
        else:
            marcrecord.add("100", a=creator)

    # Titel
    f245 = xmlrecord["metadata"]["oai_dc:dc"]["dc:title"]
    marcrecord.add("245", a=f245)

    # Erscheinungsvermerk
    if xmlrecord["metadata"]["oai_dc:dc"].get("dc:publisher"):
        f260b = xmlrecord["metadata"]["oai_dc:dc"]["dc:publisher"]
    else:
        f260b = ""

    if xmlrecord["metadata"]["oai_dc:dc"].get("dc:date"):
        f260c = xmlrecord["metadata"]["oai_dc:dc"]["dc:date"]
    else:
        f260c = ""

    publisher = ["b", f260b, "c", f260c]
    marcrecord.add("260", subfields=publisher)

    # Beschreibung
    if xmlrecord["metadata"]["oai_dc:dc"].get("dc:description"):
        f520a = xmlrecord["metadata"]["oai_dc:dc"]["dc:description"]
        marcrecord.add("520", a=f520a)

    # Schlagwörter
    if xmlrecord["metadata"]["oai_dc:dc"].get("dc:subject"):
        subjects = xmlrecord["metadata"]["oai_dc:dc"]["dc:subject"]
        if isinstance(subjects, list):
            for subject in subjects:
                if not re.search("\d\d\d", subject):
                    marcrecord.add("650", a=subject)
        else:
            if not re.search("\d\d\d", subject):
                marcrecord.add("650", a=subject)

    # weitere Urheber
    if xmlrecord["metadata"]["oai_dc:dc"].get("dc:creator"):
        creators = xmlrecord["metadata"]["oai_dc:dc"]["dc:creator"]
        if isinstance(creators, list):
            for creator in creators[1:]:
                marcrecord.add("700", a=creator)

    # Zeitschrift
    sources = xmlrecord["metadata"]["oai_dc:dc"]["dc:source"]
    if isinstance(sources, list):
        for source in sources:
            if ":" in source:
                f773a = source.split("In: ")
                if len(f773a) == 2:
                    marcrecord.add("773", a=f773a[1])
                else:
                    marcrecord.add("773", a=f773a[0])
    else:
        f773a = sources.split("In: ")
        if len(f773a) == 2:
            marcrecord.add("773", a=f773a[1])
        else:
            marcrecord.add("773", a=f773a[0])

    # Link zu Datensatz und Ressource
    urls = xmlrecord["metadata"]["oai_dc:dc"]["dc:identifier"]
    for f856u in urls:
        if "https://mediarep.org" in f856u:
            marcrecord.add("856", q="text/html", _3="Link zur Ressource", u=f856u)
        elif "doi.org" in f856u:
            marcrecord.add("856", q="text/html", _3="Zitierlink (DOI)", u=f856u)

    # Medientyp
    marcrecord.add("935", b="cofz")

    # Kollektion
    collection = ["a", f001, "b", "170", "c", "sid-170-col-mediarep"]
    marcrecord.add("980", subfields=collection)

    outputfile.write(marcrecord.as_marc())

inputfile.close()
outputfile.close()
